from machine import Pin,I2C,SPI,PWM,Timer
import framebuf
import time

LCD_Width  = 240
LCD_Height = 320

#Pin definition  引脚定义
I2C_SDA = 6
I2C_SDL = 7
I2C_IRQ = 17
I2C_RST = 16

DC = 14
CS = 9
SCK = 10
MOSI = 11
MISO = 12
RST = 13
BL = 15

CST328_LCD_TOUCH_MAX_POINTS = 5
# 定义 CST328_Touch 结构体
class CST328_Touch:
    def __init__(self):
        self.points = 0
        self.coords = [{"x": 0, "y": 0, "strength": 0} for _ in range(CST328_LCD_TOUCH_MAX_POINTS)]
        self.state = 0
class Touch_Data:
    def __init__(self):
        self.x = 0
        self.y = 0
        self.points = 0

cst328_data  =  CST328_Touch()
touch_data  =  CST328_Touch()
Touch_Data_Old  =  Touch_Data()

#LCD Driver  LCD驱动
class LCD_1inch69(framebuf.FrameBuffer):
    def __init__(self): #SPI initialization  SPI初始化
        self.width = LCD_Width
        self.height = LCD_Height
        
        self.cs = Pin(CS,Pin.OUT)
        self.rst = Pin(RST,Pin.OUT)
        
        self.cs(1)
        self.spi = SPI(1,100_000_000,polarity=0, phase=0,bits= 8,sck=Pin(SCK),mosi=Pin(MOSI),miso=None)
        self.dc = Pin(DC,Pin.OUT)
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        self.init_display()
        
        #Define color, Micropython fixed to BRG format  定义颜色，Micropython固定为BRG格式
        self.red   =   0x07E0
        self.green =   0x001f
        self.blue  =   0xf800
        self.white =   0xffff
        self.black =   0x0000
        self.brown =   0X8430
        
        self.fill(self.white) #Clear screen  清屏
        self.show()#Show  显示

        self.pwm = PWM(Pin(BL))
        self.pwm.freq(5000) #Turn on the backlight  开背光
        
    def write_cmd(self, cmd): #Write command  写命令
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf): #Write data  写数据
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)
        
    def set_bl_pwm(self,duty): #Set screen brightness  设置屏幕亮度
        self.pwm.duty_u16(duty)#max 65535
        
    def init_display(self): #LCD initialization  LCD初始化
        """Initialize dispaly"""  
        self.rst(1)
        time.sleep(0.01)
        self.rst(0)
        time.sleep(0.01)
        self.rst(1)
        time.sleep(0.01)
        
        self.write_cmd(0x36)
        self.write_data(0x00)

        self.write_cmd(0x3A)
        self.write_data(0x05)

        self.write_cmd(0xB2)
        self.write_data(0x0B)
        self.write_data(0x0B)
        self.write_data(0x00)
        self.write_data(0x33)
        self.write_data(0x35)

        self.write_cmd(0xB7)
        self.write_data(0x11)

        self.write_cmd(0xBB)
        self.write_data(0x35)

        self.write_cmd(0xC0)
        self.write_data(0x2C)

        self.write_cmd(0xC2)
        self.write_data(0x01)

        self.write_cmd(0xC3)
        self.write_data(0x0D)

        self.write_cmd(0xC4)
        self.write_data(0x20)

        self.write_cmd(0xC6)
        self.write_data(0x13)

        self.write_cmd(0xD0)
        self.write_data(0xA4)
        self.write_data(0xA1)

        self.write_cmd(0xD6)
        self.write_data(0xA1)

        self.write_cmd(0xE0)
        self.write_data(0xF0)
        self.write_data(0x06)
        self.write_data(0x0B)
        self.write_data(0x0A)
        self.write_data(0x09)
        self.write_data(0x26)
        self.write_data(0x29)
        self.write_data(0x33)
        self.write_data(0x41)
        self.write_data(0x18)
        self.write_data(0x16)
        self.write_data(0x15)
        self.write_data(0x29)
        self.write_data(0x2D)

        self.write_cmd(0xE1)
        self.write_data(0xF0)
        self.write_data(0x04)
        self.write_data(0x08)
        self.write_data(0x08)
        self.write_data(0x07)
        self.write_data(0x03)
        self.write_data(0x28)
        self.write_data(0x32)
        self.write_data(0x40)
        self.write_data(0x3B)
        self.write_data(0x19)
        self.write_data(0x18)
        self.write_data(0x2A)
        self.write_data(0x2E)

        self.write_cmd(0x21)

        self.write_cmd(0x11)
        time.sleep(0.12)
        self.write_cmd(0x29)
    
    #设置窗口    
    def setWindows(self,Xstart,Ystart,Xend,Yend): 
        self.write_cmd(0x2A)
        self.write_data(Xstart >> 8)
        self.write_data(Xstart)
        self.write_data((Xend-1) >> 8)
        self.write_data(Xend-1)
        
        self.write_cmd(0x2B)
        self.write_data((Ystart) >> 8)
        self.write_data(Ystart)
        self.write_data(((Ystart)-1) >> 8)
        self.write_data((Ystart)-1)
        
        self.write_cmd(0x2C)
     
    #Show  显示   
    def show(self): 
        self.setWindows(0,0,self.width,self.height)
        
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)
        
    '''
        Partial display, the starting point of the local
        display here is reduced by 10, and the end point
        is increased by 10
    '''
    
    def Windows_show(self,Xstart,Ystart,Xend,Yend):
        if Xstart > Xend:
            data = Xstart
            Xstart = Xend
            Xend = data
            
        if (Ystart > Yend):        
            data = Ystart
            Ystart = Yend
            Yend = data
        
        if (Xend < 239):
            Xend = Xend + 1
        if (Yend < 319):
            Yend = Yend + 1
            
        self.setWindows(Xstart,Ystart,Xend,Yend)      
        self.cs(1)
        self.dc(1)
        self.cs(0)
        for i in range (Ystart,Yend):             
            Addr = (Xstart * 2) + (i * 240 * 2)                
            self.spi.write(self.buffer[Addr : Addr+((Xend-Xstart)*2)])
        self.cs(1)
        
    #Write characters, size is the font size, the minimum is 1  
    #写字符，size为字体大小,最小为1
    def write_text(self,text,x,y,size,color):
        ''' Method to write Text on OLED/LCD Displays
            with a variable font size

            Args:
                text: the string of chars to be displayed
                x: x co-ordinate of starting position
                y: y co-ordinate of starting position
                size: font size of text
                color: color of text to be displayed
        '''
        background = self.pixel(x,y)
        info = []
        # Creating reference charaters to read their values
        self.text(text,x,y,color)
        for i in range(x,x+(8*len(text))):
            for j in range(y,y+8):
                # Fetching amd saving details of pixels, such as
                # x co-ordinate, y co-ordinate, and color of the pixel
                px_color = self.pixel(i,j)
                info.append((i,j,px_color)) if px_color == color else None
        # Clearing the reference characters from the screen
        self.text(text,x,y,background)
        # Writing the custom-sized font characters on screen
        for px_info in info:
            self.fill_rect(size*px_info[0] - (size-1)*x , size*px_info[1] - (size-1)*y, size, size, px_info[2]) 
    
        
#Touch drive  触摸驱动
class Touch_CST328(object):
    #Initialize the touch chip  初始化触摸芯片
    def __init__(self,address=0x1A,mode=0,i2c_num=1,i2c_sda=I2C_SDA,i2c_scl=I2C_SDL,irq_pin=I2C_IRQ,rst_pin=I2C_RST,LCD=None):
        self._bus = I2C(id=i2c_num,scl=Pin(i2c_scl),sda=Pin(i2c_sda),freq=400_000) #Initialize I2C 初始化I2C
        self._address = address #Set slave address  设置从机地址
        self.int=Pin(irq_pin,Pin.IN, Pin.PULL_UP)         
        self.rst=Pin(rst_pin,Pin.OUT)
        self.Reset()
        bRet=self.CST328_Read_cfg()
        if bRet :
            print("Success:Detected CST328.")
        else    :
            print("Error: Not Detected CST328.")
            return None
        self.int.irq(handler=self.Int_Callback,trigger=Pin.IRQ_FALLING)
    def _read_nbyte(self, reg, length):
        # 拆分16位寄存器地址成高8位和低8位
        reg_high = (reg >> 8) & 0xFF
        reg_low = reg & 0xFF
        # 发送寄存器地址的高8位和低8位
        self._bus.writeto_mem(int(self._address), reg_high, bytes([reg_low]))
        # 读取数据
        rec = self._bus.readfrom(int(self._address), length)
        return rec
    
    def _write_null(self, cmd):
        # 拆分16位寄存器地址成高8位和低8位
        cmd_high = (cmd >> 8) & 0xFF
        cmd_low = cmd & 0xFF
        self._bus.writeto_mem(int(self._address), cmd_high, bytes([cmd_low]))
        
    def _write_nbyte(self, cmd, val):
        # 拆分16位寄存器地址成高8位和低8位
        cmd_high = (cmd >> 8) & 0xFF
        cmd_low = cmd & 0xFF
        
        # 发送寄存器地址的高8位和低8位，以及要写入的值
        self._bus.writeto_mem(int(self._address), cmd_high, bytes([cmd_low, val]))

    def CST328_Read_cfg(self):
        buf = bytearray(24) # 创建包含 24 个uint8_t类型的缓冲区
        HYN_REG_MUT_DEBUG_INFO_MODE         = 0xD101
        HYN_REG_MUT_NORMAL_MODE             = 0xD109
        HYN_REG_MUT_DEBUG_INFO_TP_NTX       = 0xD1F4
        HYN_REG_MUT_DEBUG_INFO_RES_X        = 0xD1F8
        HYN_REG_MUT_DEBUG_INFO_BOOT_TIME    = 0xD1FC

        self._write_null(HYN_REG_MUT_DEBUG_INFO_MODE)
        buf = self._read_nbyte(HYN_REG_MUT_DEBUG_INFO_BOOT_TIME, 4)
        print("TouchPad_ID:" + hex(buf[0]) + "," + hex(buf[1]) + "," + hex(buf[2]) + "," + hex(buf[3]))
        buf = self._read_nbyte(HYN_REG_MUT_DEBUG_INFO_RES_X, 4)
        print("TouchPad_X_MAX:" + str(buf[1]*256+buf[0]) + "    TouchPad_Y_MAX:" + str(buf[3]*256+buf[2]) + "\r\n")

        buf = self._read_nbyte(HYN_REG_MUT_DEBUG_INFO_TP_NTX, 24)
        print("D1F4:" + hex(buf[0]) + "," + hex(buf[1]) + "," + hex(buf[2]) + "," + hex(buf[3]))
        print("D1F8:" + hex(buf[4]) + "," + hex(buf[5]) + "," + hex(buf[6]) + "," + hex(buf[7]))
        print("D1FC:" + hex(buf[8]) + "," + hex(buf[9]) + "," + hex(buf[10]) + "," + hex(buf[11]))
        print("D200:" + hex(buf[12]) + "," + hex(buf[13]) + "," + hex(buf[14]) + "," + hex(buf[15]))
        print("D204:" + hex(buf[16]) + "," + hex(buf[17]) + "," + hex(buf[18]) + "," + hex(buf[19]))
        print("D208:" + hex(buf[20]) + "," + hex(buf[21]) + "," + hex(buf[22]) + "," + hex(buf[23]))
        print("CACA Read:" + hex((buf[11]<< 8) | buf[10]))

        self._write_null(HYN_REG_MUT_NORMAL_MODE)
        if (((buf[11] << 8) | buf[10]) != 0xCACA):
            return False
        return True

    def Int_Callback(self,pin):
        self.Touch_Read_Data()
        self.Touch_Get_XY()
        
    #Reset  复位    
    def Reset(self):
        self.rst(0)
        time.sleep_ms(1)
        self.rst(1)
        time.sleep_ms(50)
    
    #Get the coordinates of the touch  获取触摸的坐标
    def Touch_Read_Data(self):
        buf = bytearray(41) # 创建包含41个uint8_t类型的缓冲区
        ESP_LCD_TOUCH_CST328_READ_Number_REG    = 0xD005
        ESP_LCD_TOUCH_CST328_READ_XY_REG        = 0xD000   
        clear = 0
        touch_cnt = 0
        buf = self._read_nbyte( ESP_LCD_TOUCH_CST328_READ_Number_REG, 1)
        if ((buf[0] & 0x0F) == 0x00):                                               # 判断手指数量
           self._write_nbyte(ESP_LCD_TOUCH_CST328_READ_Number_REG,clear)  # No touch data
        else :
            # Count of touched points
            touch_cnt = buf[0] & 0x0F
            if (touch_cnt > CST328_LCD_TOUCH_MAX_POINTS or touch_cnt == 0) :
                self._write_nbyte(ESP_LCD_TOUCH_CST328_READ_Number_REG, clear)
                return True
            
            # Read all points 
            buf = self._read_nbyte( ESP_LCD_TOUCH_CST328_READ_XY_REG, 27)
            # Clear all 
            self._write_nbyte(ESP_LCD_TOUCH_CST328_READ_Number_REG, clear)
            
            # Number of touched points 
            if(touch_cnt > CST328_LCD_TOUCH_MAX_POINTS):
                touch_cnt = CST328_LCD_TOUCH_MAX_POINTS
            cst328_data.points = touch_cnt
            num = 0
            # Fill all coordinates 
            for i in range(touch_cnt):
                if(i>0):
                    num = 2
                cst328_data.coords[i]["x"] = ((buf[(i * 5) + 1 + num] << 4) + ((buf[(i * 5) + 3 + num] & 0xF0)>> 4))               
                cst328_data.coords[i]["y"] = ((buf[(i * 5) + 2 + num] << 4) + ( buf[(i * 5) + 3 + num] & 0x0F))
                cst328_data.coords[i]["strength"] = (buf[(i * 5) + 4 + num])
            #print("Touch : X=" + str(cst328_data.coords[0]["x"]) + "   Y=" + str(cst328_data.coords[0]["y"]) + "   points=" + str(cst328_data.points))

            
    def Touch_Get_XY(self):
        if cst328_data.points :
            for i in range(cst328_data.points):
                touch_data.coords[i]["x"] = cst328_data.coords[i]["x"]
                touch_data.coords[i]["y"] = cst328_data.coords[i]["y"]
                touch_data.coords[i]["strength"]= cst328_data.coords[i]["strength"]
            touch_data.state = 1
        else:
            touch_data.state  = 0   
        touch_data.points = cst328_data.points
        cst328_data.points = 0
        print("Touch : X=" + str(touch_data.coords[0]["x"]) + "   Y=" + str(touch_data.coords[0]["y"]) + "   points=" + str(touch_data.points))
        
    
    #Draw points and show  画点并显示  
    def Touch_HandWriting(self):
        x = y = 0
        color = 0
        
        LCD.fill(LCD.white)
        LCD.show()
        Start_Flag = 0
        try:
            while True:              
                if touch_data.state :
                    touch_data.state  = 0 
                    if(touch_data.points > 1):
                      for i in range(touch_data.points):
                        LCD.rect(touch_data.coords[i]["x"], touch_data.coords[i]["y"],30,30,LCD.red)
                      Touch_Data_Old.points = touch_data.points
                      LCD.Windows_show(x,y,LCD.width,LCD.height)
                    else:
                      if(Touch_Data_Old.points > 1) :
                        LCD.fill(LCD.white)
                        LCD.Windows_show(x,y,LCD.width,LCD.height)
                      if(Start_Flag == 0):
                        Start_Flag = 1
                        Touch_Data_Old.x = touch_data.coords[0]["x"]
                        Touch_Data_Old.y = touch_data.coords[0]["y"]
                      else:
                        LCD.line(Touch_Data_Old.x, Touch_Data_Old.y, touch_data.coords[0]["x"], touch_data.coords[0]["y"],6)
                        LCD.Windows_show(Touch_Data_Old.x,Touch_Data_Old.y,touch_data.coords[0]["x"], touch_data.coords[0]["y"])
                        Touch_Data_Old.x = touch_data.coords[0]["x"]
                        Touch_Data_Old.y = touch_data.coords[0]["y"]
                      Touch_Data_Old.points = touch_data.points
                      touch_data.points = 0
                elif(touch_data.state == 0 and touch_data.points > 1):
                    touch_data.points = 0
                    Touch_Data_Old.points = 0
                    LCD.fill(LCD.white)
                    LCD.Windows_show(x,y,LCD.width,LCD.height)
        except KeyboardInterrupt:
            pass

            
if __name__=='__main__':
  
    LCD = LCD_1inch69()
    LCD.set_bl_pwm(65535)

    Touch=Touch_CST328()

    Touch.Touch_HandWriting()


















